#
#  gpsman --- GPS Manager: a manager for GPS receiver data
#
#  Copyright (c) 2000 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
#
#    This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.
#
#  File: geod.tcl
#  Last change:  10 November 2000
#

## Most of the geodetic information herein taken or adopted from
#  gpstrans - a program to communicate with garmin gps
#      containing parts taken from John F. Waers (jfwaers@csn.net)
#      program MacGPS.
#      Copyright (c) 1995 by Carsten Tschach (tschach@zedat.fu-berlin.de)
##
#
## Some geodetic information kindly provided by
#      Luisa Bastos, Universidade do Porto
##
#
## Some more definitions taken from:
#      - the PROJ4.0 package by Gerald I. Evenden (gie@kai.er.usgs.gov)
#      - International Institute for Aerospace Survey and Earth Sciences (ITC),
#      Enschede, http://kartoweb.itc.nl/geometrics
##


proc EllipsdData {datum} {
    # yield the ellipsoid data for datum as a list with
    #    a - semi-major axis
    #    f - flattening
    #    dx, dy, dz - translation of center
    global DATUM DATELLPSD ELLPSNAME ELLPSDDEF TXT

    set id [FindArrayIndex DATUM $datum -1]
    set e $DATELLPSD($id)
    foreach ie [array names ELLPSNAME] {
	if {! [string compare "$ELLPSNAME($ie)" "$e"] } {
	    break
	}
    }
    set f [expr 1/$ELLPSDDEF($ie,invf)]
    return [list $ELLPSDDEF($ie,a) $f $DATELLPSD($id,dx) $DATELLPSD($id,dy) \
            $DATELLPSD($id,dz)]
}

# definitions of known datums and ellipsoids

###### known datums
     # when changing this list make sure you update the definitions below

proc FillDatumMenu {menu commd} {
    # fill menu with known datum names
    # see proc FillMenu (utils.tcl) for the details on the arguments
    global TXT

    FillMenu $menu "$commd" \
      [list "WGS 84" "WGS 72" \
	--- \
	@ [list meur Europe \
		"CH-1903" "ED73" "European 1950" "European 1979" \
		"Finland Hayford" "Hjorsey 1955" "Ireland 1965" "Lisboa" \
		"NTF (Nouvelle Triangulation de France)" \
		"Observatorio 1966" "Observatorio Meteorologico 1939" \
		"Ord Srvy Grt Britn" "Pico De Las Nieves" \
		"Porto Santo 1936" Potsdam "Pulkovo 1942" "Rome 1940" \
		"Rijks Driehoeksmeting" "RT 90" "S-42 (Pulkovo 1942)" \
		"S-JTSK" "Sao Braz" \
		"Southeast Base" "Southwest Base"] \
	@ [list mame America \
		"Bermuda 1957" "Bogota Observatory" "Campo Inchauspe" \
		"Cape Canaveral" "Chua Astro" "Corrego Alegre" \
		"Gan 1970" \
		"NAD27 Alaska" "NAD27 Bahamas" "NAD27 Canada" \
		"NAD27 Canal Zone" "NAD27 Caribbean" "NAD27 Central" \
		"NAD27 CONUS" "NAD27 Cuba" "NAD27 Greenland" \
		"NAD27 Mexico" "NAD27 San Salvador" "NAD83" \
		"Naparima BWI" "Old Hawaiian" "Prov So Amrican `56" \
		"Prov So Chilean `63" "Puerto Rico" "Qornoq" \
		"Reunion" "Sapper Hill 1943" "South American `69" \
		"Yacare"] \
	@ [list mafr Africa \
		"Adindan" "Afgooye" "Arc 1950" \
		"Arc 1960" "Astro DOS 71/4" "Ayabelle Lighthouse" \
		"Bissau" "Cape" "Carthage" "Dabola" "Leigon" \
		"Liberia 1964" "M'Poraloko" "Massawa" \
		"Merchich" "Minna" "North Sahara 1959" "Old Egyptian" \
		"Point 58" "Pointe Noire 1948" "Schwarzeck" \
		"Tananarive Observatory 1925" "Tristan Astro 1968" \
		"Voirol 1960" "Zanderij"] \
	@ [list mmea "Middle East" \
		"Ain el Abd 1970" "Herat North" "Nahrwn Masirah Ilnd" \
		"Nahrwn Saudi Arbia" "Nahrwn United Arab" "Oman"] \
	@ [list masi Asia \
	        "Bukit Rimpah" "Gunung Segara" "Hong Kong 1963" \
		"Hu-Tzu-Shan" "Indian Bangladesh" "Indian (India, Nepal)" \
		"Indian (Pakistan)" "Indian Thailand" "Indian 1954" \
		"Indian 1975" "Indonesian 1974" "Kandawala" "South Asia" \
		"Tokyo"] \
	@ [list manz "Australia, NZ" \
		"Australian Geod `66" "Australian Geod `84" \
		"Geodetic Datum `49"] \
	@ [list mo1 "$TXT(others) \[A-I\]" \
	        "American Samoa 1962" "Anna 1 Astro 1965" \
		"Antigua Island Astro 1943" "Ascension Island `58" \
		"Astro B4 Sorol Atoll" "Astro Beacon \"E\"" \
		"Astronomic Stn `52" "Astro Tern Island (FRIG)" \
		"Bellevue (IGN)" "Camp Area Astro" "Canton Astro 1966" \
		"Chatham 1971" "Deception Island" "Djakarta (Batavia)" \
		"DOS 1968" "Easter Island 1967" "Fort Thomas 1955" \
		"Gandajika Base" "Guam 1963" \
		"GUX 1 Astro" "ISTS 061 Astro 1968" "ISTS 073 Astro `69"] \
	@ [list mo2 "$TXT(others) \[J-Z\]" \
		"Johnston Island" "Johnston Island 1961" \
		"Kerguelen Island" "Kertau 1948" "Kusaie Astro 1951" \
		"L.C. 5 Astro" "Luzon Mindanao" "Luzon Philippines" \
		"Mahe 1971" "Marco Astro" "Midway Astro 1961" \
		"Montserrat Island Astro 1958" \
		"Pitcairn Astro 1967" "Qatar National" "Santo (DOS)" \
		"Selvagem Grande 1938" "Timbalai 1948" "Viti Levu 1916" \
		"Wake Island Astro 1952" "Wake-Eniwetok `60"]]
    return
}

set DATUM(0) "Adindan"
array set DATELLPSD {
    0  "Clarke 1880"
    0,dx -162    0,dy  -12    0,dz  206
}
             # Somalia
set DATUM(1) "Afgooye"
array set DATELLPSD {
    1  "Krassovsky 1940"
    1,dx    -43    1,dy   -163    1,dz     45 
}
set DATUM(2) "Ain el Abd 1970"
array set DATELLPSD {
    2  "International 1924"
    2,dx   -150    2,dy   -251    2,dz     -2
}
             # Cocos Islands
set DATUM(3) "Anna 1 Astro 1965"
array set DATELLPSD {
    3  "Australian National"
    3,dx   -491    3,dy    -22    3,dz    435 
}
set DATUM(4) "Arc 1950"
array set DATELLPSD {
    4  "Clarke 1880"
    4,dx   -143    4,dy    -90    4,dz   -294 
}
set DATUM(5) "Arc 1960"
array set DATELLPSD {
    5  "Clarke 1880"
    5,dx   -160    5,dy     -8    5,dz   -300 
}
set DATUM(6) "Ascension Island `58"
array set DATELLPSD {
    6  "International 1924"
    6,dx   -207    6,dy    107    6,dz     52 
}
set DATUM(7) "Astro B4 Sorol Atoll"
array set DATELLPSD {
    7  "International 1924"
    7,dx    114    7,dy   -116    7,dz   -333 
}
             # Iwo Jima
set DATUM(8) "Astro Beacon \"E\""
array set DATELLPSD {
    8  "International 1924"
    8,dx    145    8,dy     75    8,dz   -272
}
             # St Helena Island
set DATUM(9) "Astro DOS 71/4"
array set DATELLPSD {
    9  "International 1924"
    9,dx   -320    9,dy    550    9,dz   -494
}
              # Marcus Island
set DATUM(10) "Astronomic Stn `52"
array set DATELLPSD {
    10  "International 1924"
    10,dx    124    10,dy   -234    10,dz    -25 
}
set DATUM(11) "Australian Geod `66"
array set DATELLPSD {
    11  "Australian National"
    11,dx   -133    11,dy    -48    11,dz    148 
}
set DATUM(12) "Australian Geod `84"
array set DATELLPSD {
    12  "Australian National"
    12,dx   -134    12,dy    -48    12,dz    149 
}
              # Efate and Erromango Islands
set DATUM(13) "Bellevue (IGN)"
array set DATELLPSD {
    13  "International 1924"
    13,dx   -127    13,dy   -769    13,dz    472 
}
set DATUM(14) "Bermuda 1957"
array set DATELLPSD {
    14  "Clarke 1866"
    14,dx    -73    14,dy    213    14,dz    296 
}
              # Colombia
set DATUM(15) "Bogota Observatory"
array set DATELLPSD {
    15  "International 1924"
    15,dx    307    15,dy    304    15,dz   -318
}
              # Argentina
set DATUM(16) "Campo Inchauspe"
array set DATELLPSD {
    16  "International 1924"
    16,dx   -148    16,dy    136    16,dz     90 
}
              # Phoenix Islands
set DATUM(17) "Canton Astro 1966"
array set DATELLPSD {
    17  "International 1924"
    17,dx    298    17,dy   -304    17,dz   -375
}
              # South Africa
set DATUM(18) "Cape"
array set DATELLPSD {
    18  "Clarke 1880"
    18,dx   -136    18,dy   -108    18,dz   -292
}
              # Bahamas, Florida
set DATUM(19) "Cape Canaveral"
array set DATELLPSD {
    19  "Clarke 1866"
    19,dx     -2    19,dy    150    19,dz    181
}
              # Tunisia
set DATUM(20) "Carthage"
array set DATELLPSD {
    20  "Clarke 1880"
    20,dx   -263    20,dy      6    20,dz    431 
}
set DATUM(21) "CH-1903"
array set DATELLPSD {
    21  "Bessel 1841"
    21,dx    674    21,dy     15    21,dz    405
}
              # New Zealand (Chatham Island)
set DATUM(22) "Chatham 1971"
array set DATELLPSD {
    22  "International 1924"
    22,dx    175    22,dy    -38    22,dz    113 
}
              # Paraguay
set DATUM(23) "Chua Astro"
array set DATELLPSD {
    23  "International 1924"
    23,dx   -134    23,dy    229    23,dz    -29
}
              # Brazil
set DATUM(24) "Corrego Alegre"
array set DATELLPSD {
    24  "International 1924"
    24,dx   -206    24,dy    172    24,dz     -6 
}
set DATUM(25) "Djakarta (Batavia)"
array set DATELLPSD {
    25  "Bessel 1841"
    25,dx   -377    25,dy    681    25,dz    -50
}
              # New Georgia Islands (Gizo Island)
set DATUM(26) "DOS 1968"
array set DATELLPSD {
    26  "International 1924"
    26,dx    230    26,dy   -199    26,dz   -752 
}
set DATUM(27) "Easter Island 1967"
array set DATELLPSD {
    27  "International 1924"
    27,dx    211    27,dy    147    27,dz    111 
}
set DATUM(28) "European 1950"
array set DATELLPSD {
    28  "International 1924"
    28,dx    -87    28,dy    -98    28,dz   -121 
}
set DATUM(29) "European 1979"
array set DATELLPSD {
    29  "International 1924"
    29,dx    -86    29,dy    -98    29,dz   -119
}
set DATUM(30) "Finland Hayford"
array set DATELLPSD {
    30  "International 1924"
    30,dx    -78    30,dy   -231    30,dz    -97 
}
set DATUM(31) "Gandajika Base"
array set DATELLPSD {
    31  "International 1924"
    31,dx   -133    31,dy   -321    31,dz     50
}
              # New Zealand
set DATUM(32) "Geodetic Datum `49"
array set DATELLPSD {
    32  "International 1924"
    32,dx     84    32,dy    -22    32,dz    209 
}
set DATUM(33) "Guam 1963"
array set DATELLPSD {
    33  "Clarke 1866"
    33,dx   -100    33,dy   -248    33,dz    259
}
              # Guadalcanal Island
set DATUM(34) "GUX 1 Astro"
array set DATELLPSD {
    34  "International 1924"
    34,dx    252    34,dy   -209    34,dz   -751
}
              # Iceland
set DATUM(35) "Hjorsey 1955"
array set DATELLPSD {
    35  "International 1924"
    35,dx    -73    35,dy     46    35,dz    -86 
}
set DATUM(36) "Hong Kong 1963"
array set DATELLPSD {
    36  "International 1924"
    36,dx   -156    36,dy   -271    36,dz   -189 
}
set DATUM(37) "Indian Bangladesh"
array set DATELLPSD {
    37  "Everest (India 1830)"
    37,dx    289    37,dy    734    37,dz    257 
}
set DATUM(38) "Indian Thailand"
array set DATELLPSD {
    38  "Everest (India 1830)"
    38,dx    214    38,dy    836    38,dz    303 
}
set DATUM(39) "Ireland 1965"
array set DATELLPSD {
    39  "Modified Airy"
    39,dx    506    39,dy   -122    39,dz    611
}
              # South Georgia Islands
set DATUM(40) "ISTS 073 Astro `69"
array set DATELLPSD {
    40  "International 1924"
    40,dx    208    40,dy   -435    40,dz   -229 
}
set DATUM(41) "Johnston Island"
array set DATELLPSD {
    41  "International 1924"
    41,dx    191    41,dy    -77    41,dz   -204
}
              # Sri Lanka
set DATUM(42) "Kandawala"
array set DATELLPSD {
    42  "Everest (India 1830)"
    42,dx    -97    42,dy    787    42,dz     86 
}
set DATUM(43) "Kerguelen Island"
array set DATELLPSD {
    43  "International 1924"
    43,dx    145    43,dy   -187    43,dz    103
}
              # West Malaysia and Singapore
set DATUM(44) "Kertau 1948"
array set DATELLPSD {
    44  "Everest (1948)"
    44,dx    -11    44,dy    851    44,dz      5
}
              # Cayman Brac Island
set DATUM(45) "L.C. 5 Astro"
array set DATELLPSD {
    45  "Clarke 1866"
    45,dx     42    45,dy    124    45,dz    147 
}
set DATUM(46) "Liberia 1964"
array set DATELLPSD {
    46  "Clarke 1880"
    46,dx    -90    46,dy     40    46,dz     88 
}
set DATUM(47) "Luzon Mindanao"
array set DATELLPSD {
    47  "Clarke 1866"
    47,dx   -133    47,dy    -79    47,dz    -72 
}
set DATUM(48) "Luzon Philippines"
array set DATELLPSD {
    48  "Clarke 1866"
    48,dx   -133    48,dy    -77    48,dz    -51
}
              # Mahe Island
set DATUM(49) "Mahe 1971"
array set DATELLPSD {
    49  "Clarke 1880"
    49,dx     41    49,dy   -220    49,dz   -134 
}
set DATUM(50) "Marco Astro"
array set DATELLPSD {
    50  "International 1924"
    50,dx   -289    50,dy   -124    50,dz     60
}
              # Ethiopia (Eritrea)
set DATUM(51) "Massawa"
array set DATELLPSD {
    51  "Bessel 1841"
    51,dx    639    51,dy    405    51,dz     60
}
              # Morocco
set DATUM(52) "Merchich"
array set DATELLPSD {
    52  "Clarke 1880"
    52,dx     31    52,dy    146    52,dz     47
}
              # Midway Islands
set DATUM(53) "Midway Astro 1961"
array set DATELLPSD {
    53  "International 1924"
    53,dx    912    53,dy    -58    53,dz   1227
}
set DATUM(54) "Minna"
array set DATELLPSD {
    54  "Clarke 1880"
    54,dx    -92    54,dy    -93    54,dz    122 
}
set DATUM(55) "NAD27 Alaska"
array set DATELLPSD {
    55  "Clarke 1866"
    55,dx     -5    55,dy    135    55,dz    172 
}
set DATUM(56) "NAD27 Bahamas"
array set DATELLPSD {
    56  "Clarke 1866"
    56,dx     -4    56,dy    154    56,dz    178 
}
set DATUM(57) "NAD27 Canada"
array set DATELLPSD {
    57  "Clarke 1866"
    57,dx    -10    57,dy    158    57,dz    187 
}
set DATUM(58) "NAD27 Canal Zone"
array set DATELLPSD {
    58  "Clarke 1866"
    58,dx      0    58,dy    125    58,dz    201 
}
set DATUM(59) "NAD27 Caribbean"
array set DATELLPSD {
    59  "Clarke 1866"
    59,dx     -7    59,dy    152    59,dz    178 
}
set DATUM(60) "NAD27 Central"
array set DATELLPSD {
    60  "Clarke 1866"
    60,dx      0    60,dy    125    60,dz    194 
}
set DATUM(61) "NAD27 CONUS"
array set DATELLPSD {
    61  "Clarke 1866"
    61,dx     -8    61,dy    160    61,dz    176 
}
set DATUM(62) "NAD27 Cuba"
array set DATELLPSD {
    62  "Clarke 1866"
    62,dx     -9    62,dy    152    62,dz    178 
}
set DATUM(63) "NAD27 Greenland"
array set DATELLPSD {
    63  "Clarke 1866"
    63,dx     11    63,dy    114    63,dz    195 
}
set DATUM(64) "NAD27 Mexico"
array set DATELLPSD {
    64  "Clarke 1866"
    64,dx    -12    64,dy    130    64,dz    190 
}
set DATUM(65) "NAD27 San Salvador"
array set DATELLPSD {
    65  "Clarke 1866"
    65,dx      1    65,dy    140    65,dz    165 
}
set DATUM(66) "NAD83"
array set DATELLPSD {
    66  "GRS 80"
    66,dx      0    66,dy      0    66,dz      0 
}
set DATUM(67) "Nahrwn Masirah Ilnd"
array set DATELLPSD {
    67  "Clarke 1880"
    67,dx   -247    67,dy   -148    67,dz    369 
}
set DATUM(68) "Nahrwn Saudi Arbia"
array set DATELLPSD {
    68  "Clarke 1880"
    68,dx   -231    68,dy   -196    68,dz    482 
}
set DATUM(69) "Nahrwn United Arab"
array set DATELLPSD {
    69  "Clarke 1880"
    69,dx   -249    69,dy   -156    69,dz    381
}
              # Trinidad and Tobago
set DATUM(70) "Naparima BWI"
array set DATELLPSD {
    70  "International 1924"
    70,dx     -2    70,dy    374    70,dz    172 
}
set DATUM(71) "Observatorio 1966"
array set DATELLPSD {
    71  "International 1924"
    71,dx   -425    71,dy   -169    71,dz     81 
}
set DATUM(72) "Old Egyptian"
array set DATELLPSD {
    72  "Helmert 1906"
    72,dx   -130    72,dy    110    72,dz    -13 
}
set DATUM(73) "Old Hawaiian"
array set DATELLPSD {
    73  "Clarke 1866"
    73,dx     61    73,dy   -285    73,dz   -181 
}
set DATUM(74) "Oman"
array set DATELLPSD {
    74  "Clarke 1880"
    74,dx   -346    74,dy     -1    74,dz    224 
}
set DATUM(75) "Ord Srvy Grt Britn"
array set DATELLPSD {
    75  "Airy 1830"
    75,dx    375    75,dy   -111    75,dz    431
}
              # Canary Islands
set DATUM(76) "Pico De Las Nieves"
array set DATELLPSD {
    76  "International 1924"
    76,dx   -307    76,dy    -92    76,dz    127
}
              # Pitcairn Island
set DATUM(77) "Pitcairn Astro 1967"
array set DATELLPSD {
    77  "International 1924"
    77,dx    185    77,dy    165    77,dz     42 
}
set DATUM(78) "Prov So Amrican `56"
array set DATELLPSD {
    78  "International 1924"
    78,dx   -288    78,dy    175    78,dz   -376 
}
set DATUM(79) "Prov So Chilean `63"
array set DATELLPSD {
    79  "International 1924"
    79,dx     16    79,dy    196    79,dz     93
}
              # Puerto Rico, Virgin Islands
set DATUM(80) "Puerto Rico"
array set DATELLPSD {
    80  "Clarke 1866"
    80,dx     11    80,dy     72    80,dz   -101 
}
set DATUM(81) "Qatar National"
array set DATELLPSD {
    81  "International 1924"
    81,dx   -128    81,dy   -283    81,dz     22
}
              # Greenland (South)
set DATUM(82) "Qornoq"
array set DATELLPSD {
    82  "International 1924"
    82,dx    164    82,dy    138    82,dz   -189
}
              # Mascarenhas Islands
set DATUM(83) "Reunion"
array set DATELLPSD {
    83  "International 1924"
    83,dx     94    83,dy   -948    83,dz  -1262 
}
set DATUM(84) "Rome 1940"
array set DATELLPSD {
    84  "International 1924"
    84,dx   -225    84,dy    -65    84,dz      9 
}
set DATUM(85) "RT 90"
array set DATELLPSD {
    85  "Bessel 1841"
    85,dx    498    85,dy    -36    85,dz    568
}
              # Espirito Santo Island
set DATUM(86) "Santo (DOS)"
array set DATELLPSD {
    86  "International 1924"
    86,dx    170    86,dy     42    86,dz     84
}
              # Azores (Sao Miguel, Santa Maria Ids)
set DATUM(87) "Sao Braz"
array set DATELLPSD {
    87  "International 1924"
    87,dx   -203    87,dy    141    87,dz     53
}
              # East Falkland Island
set DATUM(88) "Sapper Hill 1943"
array set DATELLPSD {
    88  "International 1924"
    88,dx   -355    88,dy     16    88,dz     74
}
              # Namibia
set DATUM(89) "Schwarzeck"
array set DATELLPSD {
    89  "Bessel 1841 (Namibia)"
    89,dx    616    89,dy     97    89,dz   -251 
}
set DATUM(90) "South American `69"
array set DATELLPSD {
    90  "South American 1969"
    90,dx    -57    90,dy      1    90,dz    -41
}
              # Singapore
set DATUM(91) "South Asia"
array set DATELLPSD {
    91  "Modified Fischer 1960"
    91,dx      7    91,dy    -10    91,dz    -26 
}
set DATUM(92) "Southeast Base"
array set DATELLPSD {
    92  "International 1924"
    92,dx   -499    92,dy   -249    92,dz    314 
}
set DATUM(93) "Southwest Base"
array set DATELLPSD {
    93  "International 1924"
    93,dx   -104    93,dy    167    93,dz    -38
}
              # Brunei, E. Malaysia (Sabah Sarawak)
set DATUM(94) "Timbalai 1948"
array set DATELLPSD {
    94  "Everest (India 1830)"
    94,dx   -689    94,dy    691    94,dz    -46 
}
set DATUM(95) "Tokyo"
array set DATELLPSD {
    95  "Bessel 1841"
    95,dx   -128    95,dy    481    95,dz    664
}
              # Tristao da Cunha
set DATUM(96) "Tristan Astro 1968"
array set DATELLPSD {
    96  "International 1924"
    96,dx   -632    96,dy    438    96,dz   -609
}
              # Fiji (Viti Levu Island)
set DATUM(97) "Viti Levu 1916"
array set DATELLPSD {
    97  "Clarke 1880"
    97,dx     51    97,dy    391    97,dz    -36
}
              # Marshall Islands
set DATUM(98) "Wake-Eniwetok `60"
array set DATELLPSD {
    98  "Hough 1960"
    98,dx    101    98,dy     52    98,dz    -39 
}
set DATUM(99) "WGS 72"
array set DATELLPSD {
    99  "WGS 72"
    99,dx      0    99,dy      0    99,dz      5 
}
set DATUM(100) "WGS 84"
array set DATELLPSD {
    100  "WGS 84"
    100,dx      0    100,dy      0    100,dz      0
}
               # Suriname
set DATUM(101) "Zanderij"
array set DATELLPSD {
    101  "International 1924"
    101,dx   -265    101,dy    120    101,dz   -358 
}
set DATUM(102) "ED73"
array set DATELLPSD {
    102  "WGS 84"
    102,dx    238    102,dy    -87    102,dz    -27
}
set DATUM(103) "Lisboa"
array set DATELLPSD {
    103  "WGS 84"
    103,dx  183.544    103,dy  -46.902    103,dz  -24.135
}
set DATUM(104) "Potsdam"
array set DATELLPSD {
    104  "Bessel 1841"
    104,dx  606    104,dy  23    104,dz  413
}
               # American Samoa Islands
set DATUM(105) "American Samoa 1962"
array set DATELLPSD {
    105  "Clarke 1866"
    105,dx  -115    105,dy  118    105,dz  426
}
                 # Antigua (Leeward Islands)
set DATUM(106) "Antigua Island Astro 1943"
array set DATELLPSD {
    106  "Clarke 1880"
    106,dx -270   106,dy 13    106,dz 62
}
set DATUM(107) "Astro Tern Island (FRIG)"
array set DATELLPSD {
    107  "International 1924"
    107,dx 114    107,dy -116  107,dz -333
}
               # Djibouti
set DATUM(108) "Ayabelle Lighthouse"
array set DATELLPSD {
    108  "Clarke 1880"
    108,dx -79    108,dy -129  108,dz -145
}
               # Guinea-Bissau
set DATUM(109) "Bissau"
array set DATELLPSD {
    109  "International 1924"
    109,dx -173   109,dy 253   109,dz 27
}
               # Indonesia (Bangka and Belitung Ids)
set DATUM(110) "Bukit Rimpah"
array set DATELLPSD {
    110  "Bessel 1841"
    110,dx -384   110,dy 664   110,dz -48
}
               # Antarctica (McMurdo Camp Area)
set DATUM(111) "Camp Area Astro"
array set DATELLPSD {
    111  "International 1924"
    111,dx -104   111,dy -129   111,dz 239
}
               # Guinea
set DATUM(112) "Dabola"
array set DATELLPSD {
    112  "Clarke 1880"
    112,dx -83   112,dy 37   112,dz 124
}
               # Deception Island, Antarctica
set DATUM(113) "Deception Island"
array set DATELLPSD {
    113  "Clarke 1880"
    113,dx 260   113,dy 12   113,dz -147
}
               # Nevis, St. Kitts (Leeward Islands)
set DATUM(114) "Fort Thomas 1955"
array set DATELLPSD {
    114  "Clarke 1880"
    114,dx -7   114,dy 215   114,dz 225
}
               # Republic of Maldives
set DATUM(115) "Gan 1970"
array set DATELLPSD {
    115  "International 1924"
    115,dx -133   115,dy -321   115,dz 50
}
               # Indonesia (Kalimantan)
set DATUM(116) "Gunung Segara"
array set DATELLPSD {
    116  "Bessel 1841"
    116,dx -403   116,dy 684   116,dz 41
}
               # Afghanistan
set DATUM(117) "Herat North"
array set DATELLPSD {
    117  "International 1924"
    117,dx -333   117,dy -222   117,dz 114
}
               # Taiwan
set DATUM(118) "Hu-Tzu-Shan"
array set DATELLPSD {
    118  "International 1924"
    118,dx -637   118,dy -549   118,dz -203
}
               # India, Nepal
set DATUM(119) "Indian (India, Nepal)"
array set DATELLPSD {
    119  "Everest (India 1956)"
    119,dx 295   119,dy 736   119,dz 257
}
               # Pakistan
set DATUM(120) "Indian (Pakistan)"
array set DATELLPSD {
    120  "Everest (Pakistan)"
    120,dx 283   120,dy 682   120,dz 231
}
               # Thailand
set DATUM(121) "Indian 1954"
array set DATELLPSD {
    121  "Everest (India 1830)"
    121,dx 217   121,dy 823   121,dz 299
}
               # Thailand
set DATUM(122) "Indian 1975"
array set DATELLPSD {
    122  "Everest (India 1830)"
    122,dx 209   122,dy 818   122,dz 290
}
               # Indonesia
set DATUM(123) "Indonesian 1974"
array set DATELLPSD {
    123  "Indonesian 1974"
    123,dx -24   123,dy -15   123,dz 5
}
               # South Georgia Islands
set DATUM(124) "ISTS 061 Astro 1968"
array set DATELLPSD {
    124  "International 1924"
    124,dx -794   124,dy 119   124,dz -298
}
               # Johnston Island
set DATUM(125) "Johnston Island 1961"
array set DATELLPSD {
    125  "International 1924"
    125,dx 189   125,dy -79   125,dz -202
}
               # Caroline Islands
set DATUM(126) "Kusaie Astro 1951"
array set DATELLPSD {
    126  "International 1924"
    126,dx 647   126,dy 1777   126,dz -1124
}
               # Ghana
set DATUM(127) "Leigon"
array set DATELLPSD {
    127  "Clarke 1880"
    127,dx -130   127,dy 29   127,dz 364
}
               # Gabon
set DATUM(128) "M'Poraloko"
array set DATELLPSD {
    128  "Clarke 1880"
    128,dx -74   128,dy -130   128,dz 42
}
               # Montserrat (Leeward Islands)
set DATUM(129) "Montserrat Island Astro 1958"
array set DATELLPSD {
    129  "Clarke 1880"
    129,dx 174   129,dy 359   129,dz 365
}
               # Algeria
set DATUM(130) "North Sahara 1959"
array set DATELLPSD {
    130  "Clarke 1880"
    130,dx -186   130,dy -93   130,dz 310
}
               # France (incl. Corsica)
set DATUM(131) "NTF (Nouvelle Triangulation de France)"
array set DATELLPSD {
    131  "Clarke 1880"
    131,dx -168   131,dy -60   131,dz 320
}
               # Azores (Corvo and Flores Islands)
set DATUM(132) "Observatorio Meteorologico 1939"
array set DATELLPSD {
    132  "International 1924"
    132,dx -425   132,dy -169   132,dz 81
}
               # Mean for Burkina Faso and Niger
set DATUM(133) "Point 58"
array set DATELLPSD {
    133  "Clarke 1880"
    133,dx -106   133,dy -129   133,dz 165
}
               # Congo
set DATUM(134) "Pointe Noire 1948"
array set DATELLPSD {
    134  "Clarke 1880"
    134,dx -148   134,dy 51   134,dz -291
}
               # Porto Santo, Madeira Islands
set DATUM(135) "Porto Santo 1936"
array set DATELLPSD {
    135  "International 1924"
    135,dx -499   135,dy -249   135,dz 314
}
               # Russia
set DATUM(136) "Pulkovo 1942"
array set DATELLPSD {
    136  "Krassovsky 1940"
    136,dx 28   136,dy -130   136,dz -95
}
               # Netherlands
set DATUM(137) "Rijks Driehoeksmeting"
array set DATELLPSD {
    137  "Bessel 1841"
    137,dx -593   137,dy -26   137,dz -478
}
               # Hungary
set DATUM(138) "S-42 (Pulkovo 1942)"
array set DATELLPSD {
    138  "Krassovsky 1940"
    138,dx 28   138,dy -121   138,dz -77
}
               # Czechoslavakia (Prior 1 JAN 1993)
set DATUM(139) "S-JTSK"
array set DATELLPSD {
    139  "Bessel 1841"
    139,dx 589   139,dy 76   139,dz 480
}
               # Salvage Islands
set DATUM(140) "Selvagem Grande 1938"
array set DATELLPSD {
    140  "International 1924"
    140,dx -289   140,dy -124   140,dz 60
}
               # Madagascar
set DATUM(141) "Tananarive Observatory 1925"
array set DATELLPSD {
    141  "International 1924"
    141,dx -189   141,dy -242   141,dz -91
}
               # Algeria
set DATUM(142) "Voirol 1960"
array set DATELLPSD {
    142  "Clarke 1880"
    142,dx -123   142,dy -206   142,dz 219
}
               # Wake Atoll
set DATUM(143) "Wake Island Astro 1952"
array set DATELLPSD {
    143  "International 1924"
    143,dx 276   143,dy -57   143,dz 149
}
               # Uruguay
set DATUM(144) "Yacare"
array set DATELLPSD {
    144  "International 1924"
    144,dx -155   144,dy 171   144,dz 37
}


                  # used in Great Britain:
set ELLPSNAME(0) "Airy 1830"
set ELLPSDDEF(0,a) 6377563.396
set ELLPSDDEF(0,invf) 299.3249646	
set ELLPSNAME(1) "Modified Airy"
set ELLPSDDEF(1,a) 6377340.189
set ELLPSDDEF(1,invf) 299.3249646  	
set ELLPSNAME(2) "Australian National"
set ELLPSDDEF(2,a) 6378160.0
set ELLPSDDEF(2,invf)   298.25 
                  # used in Germany, Japan:      	
set ELLPSNAME(3) "Bessel 1841"
set ELLPSDDEF(3,a) 6377397.155
set ELLPSDDEF(3,invf) 299.1528128 
                  # used in North America: 	
set ELLPSNAME(4) "Clarke 1866"
set ELLPSDDEF(4,a) 6378206.4
set ELLPSDDEF(4,invf)   294.9786982
                  # used in France, Africa:  	
set ELLPSNAME(5) "Clarke 1880"
set ELLPSDDEF(5,a) 6378249.145
set ELLPSDDEF(5,invf) 293.465      
                  # used in India:	
set ELLPSNAME(6) "Everest (India 1830)"
set ELLPSDDEF(6,a) 6377276.345
set ELLPSDDEF(6,invf)       300.8017
set ELLPSNAME(7) "Everest (1948)"
set ELLPSDDEF(7,a) 6377304.063
set ELLPSDDEF(7,invf)             300.8017
set ELLPSNAME(8) "Modified Fischer 1960"
set ELLPSDDEF(8,a) 6378155.0
set ELLPSDDEF(8,invf)   298.3        	
set ELLPSNAME(9) "Everest (Pakistan)"
set ELLPSDDEF(9,a) 6377309.613
set ELLPSDDEF(9,invf)         300.8017
set ELLPSNAME(10) "Indonesian 1974"
set ELLPSDDEF(10,a) 6378160.0
set ELLPSDDEF(10,invf)   298.247
                  # used in Australia:
set ELLPSNAME(11) "GRS 80"
set ELLPSDDEF(11,a) 6378137.0
set ELLPSDDEF(11,invf)   298.257222101	
set ELLPSNAME(12) "Helmert 1906"
set ELLPSDDEF(12,a) 6378200.0
set ELLPSDDEF(12,invf)   298.3        	
set ELLPSNAME(13) "Hough 1960"
set ELLPSDDEF(13,a) 6378270.0
set ELLPSDDEF(13,invf)   297.0
                  # used in Europe:        	
set ELLPSNAME(14) "International 1924"
set ELLPSDDEF(14,a) 6378388.0
set ELLPSDDEF(14,invf)   297.0
                  # used in Russia: (same as Krassovsky 1942)
set ELLPSNAME(15) "Krassovsky 1940"
set ELLPSDDEF(15,a) 6378245.0
set ELLPSDDEF(15,invf)   298.3        	
set ELLPSNAME(16) "South American 1969"
set ELLPSDDEF(16,a) 6378160.0
set ELLPSDDEF(16,invf)   298.25       	
set ELLPSNAME(17) "Everest (Malaysia 1969)"
set ELLPSDDEF(17,a) 6377295.664
set ELLPSDDEF(17,invf)    300.8017
set ELLPSNAME(18) "Everest (Sabah Sarawak)"
set ELLPSDDEF(18,a) 6377298.556
set ELLPSDDEF(18,invf)    300.8017
                  # used in USA/DoD:
set ELLPSNAME(19) "WGS 72"
set ELLPSDDEF(19,a) 6378135.0
set ELLPSDDEF(19,invf)   298.26	
set ELLPSNAME(20) "WGS 84"
set ELLPSDDEF(20,a) 6378137.0
set ELLPSDDEF(20,invf)   298.257223563	
set ELLPSNAME(21) "Bessel 1841 (Namibia)"
set ELLPSDDEF(21,a) 6377483.865
set ELLPSDDEF(21,invf)      299.1528128
set ELLPSNAME(22) "Everest (India 1956)"
set ELLPSDDEF(22,a) 6377301.243
set ELLPSDDEF(22,invf)       300.8017
set ELLPSNAME(23) "Bessel (Portugal)"
set ELLPSDDEF(23,a) 6377397.155
set ELLPSDDEF(23,invf) 297.152818860
set ELLPSNAME(24) "Fischer 1960 (Mercury)"
set ELLPSDDEF(24,a) 6378166
set ELLPSDDEF(24,invf)   298.3        	
set ELLPSNAME(25) "Fischer 1968"
set ELLPSDDEF(25,a) 6378150
set ELLPSDDEF(25,invf) 298.3
                  # used in Australia:
set ELLPSNAME(26) "GRS 67"
set ELLPSDDEF(26,a) 6378160
set ELLPSDDEF(26,invf)   298.247167427
set ELLPSNAME(27) "GRS 75"
set ELLPSDDEF(27,a) 6378140
set ELLPSDDEF(27,invf)   298.257
set ELLPSNAME(28) "Hough 1956"
set ELLPSDDEF(28,a) 6378270
set ELLPSDDEF(28,invf)   297
set ELLPSNAME(29) "WGS 60"
set ELLPSDDEF(29,a) 6378165
set ELLPSDDEF(29,invf)   298.3
                  # used in USA/DoD:
set ELLPSNAME(30) "WGS 66"
set ELLPSDDEF(30,a) 6378145
set ELLPSDDEF(30,invf)   298.25

## taken from PROJ4.0

set ELLPSNAME(100) "Appl. Physics. 1965"
set ELLPSDDEF(100,a) 6378137
set ELLPSDDEF(100,invf) 298.25
set ELLPSNAME(101) "Comm. des Poids et Mesures 1799"
set ELLPSDDEF(101,a) 6375738.7
set ELLPSDDEF(101,invf) 334.29
set ELLPSNAME(102) "IAU 1976"
set ELLPSDDEF(102,a) 6378140
set ELLPSDDEF(102,invf) 298.257
set ELLPSNAME(103) "MERIT 1983"
set ELLPSDDEF(103,a) 6378137
set ELLPSDDEF(103,invf) 298.257
                  # Naval Weapons Lab., 1965
set ELLPSNAME(104) "NWL9D"
set ELLPSDDEF(104,a) 6378145
set ELLPSDDEF(104,invf) 298.25
set ELLPSNAME(105) "Southeast Asia"
set ELLPSDDEF(105,a) 6378155
set ELLPSDDEF(105,invf) 298.3
                  # Soviet Geodetic System 85
set ELLPSNAME(106) "SGS85"
set ELLPSDDEF(106,a) 6378136
set ELLPSDDEF(106,invf) 298.257
                  # used in Denmark, Iceland:
set ELLPSNAME(107) "Andrae 1876"
set ELLPSDDEF(107,a) 6377104.43
set ELLPSDDEF(107,invf) 300
                  # used in Belgium:
set ELLPSNAME(108) "Delambre 1810"
set ELLPSDDEF(108,a) 6376428
set ELLPSDDEF(108,invf) 311.5
set ELLPSNAME(109) "Engelis 1985"
set ELLPSDDEF(109,a) 6378136.05
set ELLPSDDEF(109,invf) 298.2566
set ELLPSNAME(110) "Kaula 1961"
set ELLPSDDEF(110,a) 6378163
set ELLPSDDEF(110,invf) 298.24
set ELLPSNAME(111) "Lerch 1979"
set ELLPSDDEF(111,a) 6378139
set ELLPSDDEF(111,invf) 298.257
set ELLPSNAME(112) "Maupertius 1738"
set ELLPSDDEF(112,a) 6397300
set ELLPSDDEF(112,invf) 191
                  # used in France
set ELLPSNAME(113) "Plessis 1817"
set ELLPSDDEF(113,a) 6376523
set ELLPSDDEF(113,invf) 308.640997
set ELLPSNAME(114) "Walbeck"
set ELLPSDDEF(114,a) 6376896
set ELLPSDDEF(114,invf) 302.780


