#
#  gpsman --- GPS Manager: a manager for GPS receiver data
#
#  Copyright (c) 1999 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
#
#    This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program.
#
#  File: gpsinfo.tcl
#  Last change:  9 December 2000
#
# Includes contributions by Brian Baulch (baulchb@onthenet.com.au)
#  marked "BSB contribution"


proc DisableGPS {} {
    # change interface appearance when detecting that the receiver went offline
    global RcW GPSState GPSLabel COLOUR TXT RECTYPES

    set GPSState offline ; set GPSLabel $TXT(offline)
    $RcW.frgps.bs.state configure -fg $COLOUR(offline)
    $RcW.frgps.bs.off configure -state disabled
    foreach i $RECTYPES {
	$RcW.frget.frget2.get$i configure -state disabled
	# BSB contribution: no IC put button
	if { [string compare $i "IC"] } {
	    $RcW.frput.put$i configure -state disabled
	}
    }
    return
}

proc EnableGPS {} {
    # change interface appearance when detecting that the receiver went on-line
    global RcW GPSState GPSLabel COLOUR ListInds TXT RECTYPES MYGPS

    set GPSState online ; set GPSLabel $TXT(online)
    $RcW.frgps.bs.state configure -fg $COLOUR(online)
    $RcW.frgps.bs.off configure -state normal
    foreach i $RECTYPES {
	if { [string compare $i GR] || [string compare $ListInds(GR) ""] } {
	    $RcW.frget.frget2.get$i configure -state normal
	}
	# based on BSB contribution: no IC put button
	if { [string compare $i IC] && [string compare $ListInds($i) ""] } {
		$RcW.frput.put$i configure -state normal
	}
    }
    InitGPS
    return
}

proc CheckGPS {} {
    # check connectivity to receiver
    global GPSState COLOUR MESS

    SetCursor . watch
    if {! [string compare $GPSState offline] } {
	GPSConnection EnableGPS
    } else {
	GPSConnection [list GMMessage $MESS(GPSok)] DisableGPS
    }
    ResetCursor .
    return
}

proc TurnOff {} {
    global GPSState COLOUR

    GPSOff
    DisableGPS
    return
}

proc PutOnGPS {how wh} {
    # put data into receiver
    #  $how in {all, select}, $wh in $RECTYPES
    global ListInds RECTYPES MESS TXT

    switch $how {
	all {
	    set ixs $ListInds($wh)
	}
	select {
	    if {! [string compare [set ixs [ChooseItems $wh]] ""] } {
		return
	    }
	}
    }
    switch $wh {
	GR {
	    if {! [string compare $how all] } {
		set whs $RECTYPES
	    } else {
		set ts ""
		foreach k $RECTYPES { lappend ts $TXT(name$k) }
		set whs [GMChooseFrom $MESS(putwhat) 6 $ts $RECTYPES]
	    }
	    if { [set i [lsearch $whs GR]] != -1 } {
		set whs [lreplace $whs $i $i]
		set r 1
	    } else { set r 0 }
	    foreach wh $whs {
		if { [string compare [set l [GRsElements $ixs $r $wh]] ""] } {
		    PutItemsOnGPS $wh $l
		}
	    }
	}
	default {
	    PutItemsOnGPS $wh $ixs
	}
    }
    return
}

proc PutItemsOnGPS {wh ixs} {
    # put items of kind $wh (in $RECTYPES except GR) on receiver
    #  $ixs is list of indices
    global MAX MESS TXT

    if { [llength $ixs] > $MAX($wh) } {
	GMMessage [format $MESS(toomany) $TXT(name$wh) $MAX($wh)]
	return
    }
    PutGPS $wh $ixs
    return
}

proc GetFromGPS {wh} {
    # get data from receiver
    #  $wh in $RECTYPES
    # if $wh==GR, select item types (in $RECTYPES except TR) to get
    global RECTYPES
    # BSB contribution
    global MYGPS

    if {! [string compare $wh GR] } {
	# based on BSB contribution: do not get ICs
	if { $MYGPS != "Lowrance" } {
	    set not "TR"
	} else { set not "IC TR" }
	InputToGR $RECTYPES $not Ignore GetGPSIn Ignore receiver
    } else {
	GetGPS $wh
    }
    return
}

